@@ -42,24 +42,31 @@ def _get_api_key():
     return dmi_api_key
 
 
+# Functions dealing with microepoch data
+from seedftw.base.timeseries import (
+    format_to_timetable,
+    datetime_to_microepoch,
+    microepoch_to_datetime_index,
+    microepoch_to_local_datetime,
+    synchronise,
+    split_data_loading_range,
+)
+
+
 def _datetime_to_microepoch(datetimes2use):
-    return int(datetimes2use.timestamp() * 1e6)
+    return datetime_to_microepoch(datetimes2use)
 
 
 def _microepoch_to_datetime_index(microepoch):
-    t = pd.DatetimeIndex(np.round(microepoch).astype("datetime64[us]"))
-    # t = t.replace(tzinfo=pytz.UTC)
-    return t.tz_localize("UTC")
+    return microepoch_to_datetime_index(microepoch)
 
 
 def _microepoch_to_local_datetime(microepoch):
-    t = pd.Datetime(np.round(microepoch).astype("datetime64[us]"))
-    # t = t.replace(tzinfo=pytz.UTC)
-    return t.tz_localize("Europe/Copenhagen")
+    return microepoch_to_local_datetime(microepoch)
 
 
 def __format_data_table(data):
-    return data.rename(columns={"Minutes5UTC": "t"}).set_index("t")
+    return format_to_timetable(data, time_column="Minutes5UTC", column_dict=None)
 
 
 # Loading data from DMI's API
@@ -82,6 +89,73 @@ def __dmi_api_call(address):
     return result
 
 
+# Loading data from the API for 1 parameter
+def __load_parameter_timeseries(station, parameter, start_date, end_date, resolution):
+    """
+    Documentation of the details of the data is available here:
+    >>  https://confluence.govcloud.dk/pages/viewpage.action?pageId=26476616
+    """
+
+    if isinstance(station, str):
+        station_string = station
+    else:
+        station_string = str(station).zfill(5)
+
+    parameter2id = {
+        "ambient_temperature": "temp_dry",
+        "wind_speed": "wind_speed",
+        "wind_direction": "wind_dir",
+        "relative_humidity": "humidity",
+        "global_solar_radiation": "radia_glob",
+    }
+
+    # Adding a query size limiter to approx. 6 months
+    limit2use = 30000
+    max_data_length = timedelta(weeks=24)
+    date_steps_range = split_data_loading_range(
+        start_date, end_date, step=max_data_length
+    )
+    data = None
+    for start_j, end_j in date_steps_range:
+
+        start_microepoch = _datetime_to_microepoch(start_j)
+        end_microepoch = _datetime_to_microepoch(end_j - timedelta(seconds=1))
+        query_url = "metObs/v1/observation?from={}&to={}&stationId={}&parameterId={}&limit={}".format(
+            str(start_microepoch),
+            str(end_microepoch),
+            station_string,
+            parameter2id[parameter],
+            str(limit2use),
+        )
+
+        dmi_raw_data = __dmi_api_call(query_url)
+        dmi_raw_data["t"] = _microepoch_to_datetime_index(dmi_raw_data["timeObserved"])
+        data_j = format_to_timetable(
+            dmi_raw_data[["t", "value"]],
+            time_column="t",
+            column_dict={"value": parameter},
+        )
+        if data is None:
+            data = data_j
+        else:
+            data = data.append(data_j)
+
+    if resolution == "raw" or data is None:
+        # Do nothing
+        None
+    elif resolution == "hour":
+        data = data.resample("H").mean()
+    elif resolution == "day":
+        # Averaging per day is made in local timezone
+        data.index = data.index.tz_convert("Europe/Copenhagen")
+        data = data.resample("D").mean()
+        data.index = data.index.tz_convert("UTC")
+    else:
+        print("Error: invalid resolution:" + resolution)
+
+    return data
+
+
 @lru_cache(maxsize=1)
 def get_weather_stations():
     """Loads the list of the Danish weather stations
@@ -114,6 +188,7 @@ def get_weather_station_data(
     start_date=(datetime.now() - timedelta(days=3)),
     end_date=datetime.now(),
     resolution="hour",
+    parameters=["ambient_temperature"],
 ):
     """Loads historical data for a given weather station
 
@@ -127,6 +202,10 @@ def get_weather_station_data(
         Time for the end of the historical period
     resolution: str
         Resolution of the data to load (raw,hour,day)
+    parameters : list(str)
+        List of parameters to be loaded
+        (Options: "ambient_temperature","wind_speed","wind_direction",
+         "relative_humidity","global_solar_radiation")
 
     Raises
     ------
@@ -136,6 +215,11 @@ def get_weather_station_data(
     -------
     data : pandas.DataFrame(t,ambient_temperature)
         Timetable of the ambient temperature [degrees C]
+
+
+    Documentation of the details of the data is available here:
+    >>  https://confluence.govcloud.dk/pages/viewpage.action?pageId=26476616
+
     """
 
     if isinstance(station, str):
@@ -143,36 +227,16 @@ def get_weather_station_data(
     else:
         station_string = str(station).zfill(5)
 
-    start_microepoch = _datetime_to_microepoch(start_date)
-    end_microepoch = _datetime_to_microepoch(end_date)
-    limit2use = 30000
-    query_url = "metObs/v1/observation?from={}&to={}&stationId={}&parameterId=temp_dry&limit={}".format(
-        str(start_microepoch),
-        str(end_microepoch),
-        station_string,
-        str(limit2use),
-    )
+    data = None
 
-    dmi_raw_data = __dmi_api_call(query_url)
-    t = _microepoch_to_datetime_index(dmi_raw_data["timeObserved"])
-    t.name = "t"
-
-    data = pd.DataFrame({"ambient_temperature": dmi_raw_data["value"]})
-    data.index = t
-    data = data.sort_index(ascending=True)
-
-    if resolution == "raw":
-        # Do nothing
-        None
-    elif resolution == "hour":
-        data = data.resample("H").mean()
-    elif resolution == "day":
-        # Averaging per day is made in local timezone
-        data.index = data.index.tz_convert("Europe/Copenhagen")
-        data = data.resample("D").mean()
-        data.index = data.index.tz_convert("UTC")
-    else:
-        print("Error: invalid resolution:" + resolution)
+    for par_i in parameters:
+        data_i = __load_parameter_timeseries(
+            station, par_i, start_date, end_date, resolution
+        )
+        if data is None:
+            data = data_i
+        else:
+            data = synchronise(data, data_i, "union")
 
     return data
 
@@ -217,11 +281,12 @@ def get_closest_weather_station(latitude, longitude):
 
 
 def get_data_for_coordinates(
-    latitude,
-    longitude,
+    latitude=57.046707,
+    longitude=9.935932,
     start_date=(datetime.now() - timedelta(days=3)),
     end_date=datetime.now(),
     resolution="hour",
+    parameters=["ambient_temperature"],
 ):
     """Loads historical data from the closest weather station to a set of coordinates
 
@@ -237,6 +302,10 @@ def get_data_for_coordinates(
         Time for the end of the historical period
     resolution: str
         Resolution of the data to load (raw,hour,day)
+    parameters : list(str)
+        List of parameters to be loaded
+        (Options: "ambient_temperature","wind_speed","wind_direction",
+         "relative_humidity","global_solar_radiation")
 
     Raises
     ------
@@ -254,5 +323,6 @@ def get_data_for_coordinates(
         start_date=start_date,
         end_date=end_date,
         resolution=resolution,
+        parameters=parameters,
     )
     return data
